#ifndef INCLUDE_CGRABTHREAD_H
#define INCLUDE_CGRABTHREAD_H

// Include files to use the PYLON API
#include <pylon/PylonIncludes.h>
#include <pylon/Synch.h>
#include <pylon/gige/BaslerGigECamera.h>

// Include to use posix threads
#include "pthread.h"

typedef Pylon::CBaslerGigECamera Camera_t;

struct ThreadParams_t
{
    std::ostringstream         *pDescription;
    Camera_t                   *pCamera;
    Pylon::WaitObjectEx        *pReadyToGo;
    Pylon::WaitObjectEx        *pGo;
    uint32_t                    maxQueueBuffer;
    uint32_t                    ImagesToGrab;
    Pylon::CLock                *pLock;
};

class CGrabThread 
{
    public:
        CGrabThread( ThreadParams_t ThreadParams );
        virtual ~CGrabThread();
        virtual void Create();
    
        operator pthread_t&() { return m_Thread; }

    protected:
        /// static thread function caller
        static void * ThreadMain(void * lpData)
        {
            // call the run function within the object context
            CGrabThread *This = (CGrabThread*)lpData;
            return This->Run();
        }

        void* Run();

    private:
        uint32_t Grab( const uint32_t numberOfImagesToGrab, const uint32_t maxQueueBuffer, Camera_t::StreamGrabber_t* pStreamGrabber);

    protected:
        pthread_t    m_Thread;  // wrapper for the thread handle
        ThreadParams_t m_Tp;
};

#endif
